use Getopt::Long;
use Net::NNTP;

$SIG{'INT'} = $SIG{'QUIT'} = \&clean_exit;

my ($newgroup, $debug, $oldgroup, $server, $usage, $annotate);
my ($force, $directory, $recurse);

GetOptions("n=s" => \$newgroup, 
           "g"   => \$debug, 
           "r=s" => \$oldgroup, 
           "s=s" => \$server, 
           "h"   => \$usage, 
           "a"   => \$annotate, 
           "d=s" => \$directory, 
           "c"   => \$recurse, 
           "f"   => \$force
           );

print_usage() if $usage;

$annotate = 1 if $debug;
$directory = $directory || "./";
$server = $server || "news.foo.bar";

my $nntp = Net::NNTP->new($server, Debug => $debug ? 1 : 0);
die "Couldn't open connection" unless $nntp;

my (%groups);

init_group_table();

init_new_group($newgroup) if $newgroup;
delete $groups{$oldgroup} if $oldgroup;

foreach my $group (sort keys %groups)
{
    get_group_mesgs($group, $groups{$group}, $directory, $recurse);
}

update_group_table();

sub get_group_mesgs
{
    my ($group, $first, $directory, $recurse) = @_;
    my @groupinfo = $nntp->group($group);
    my $last=$groupinfo[2];
    
    print "Retrieving $group (messages $first to $last)\n" 
        if $annotate;
    
    if (($recurse) && ($last-$first++))
    {
        mkdir ("$directory/$group", 0700) 
            unless (-d "$directory/$group");
    }
    
    for (my $id=($first);$id<=$last;$id++)
    {
        print "Getting article $id\n" if $annotate;
        my $lines = $nntp->article($id);
        next unless $lines;
        my ($filespec) = $recurse ? ">$directory/$group/$id" :
            ">$directory/$group.$id";
        
        if (open(D, $filespec))
        {
            print D @$lines;
            close(D);
        }
        else
        {
            warn "Couldn't create $filespec\n";
        }
        $groups{$group} = $id;
    }
}

sub init_new_group
{
    my ($group) = @_;
    
    if (exists($groups{$group}))
    {
        warn "$group is already in table\n" if $annotate;
        return unless ($force);
        warn "Resetting $group\n" if $annotate;
    }
    my @groupinfo = $nntp->group($group);
    if (@groupinfo)
    {
        $groups{$group} = $groupinfo[1]-1;
        print "Added $group starting msgno $groupinfo[1]\n" 
            if $annotate;
    }
    else
    {
        warn "Group $group is not available on this server";
    }
}

sub init_group_table
{
    unless (open(GROUPS, "<autonntp.rc"))
    {
        warn "Couldn't open an existing group file\n";
        warn "A new file will be created\n";
        return;
    }
    while(<GROUPS>)
    {
        chomp;
        my ($group, $lastid) = split ', ';
        $groups{$group} = $lastid;
    }
    close(GROUPS);
}

sub update_group_table
{
    
    open(GROUPS, ">autonntp.rc") 
        || die "Couldn't update the group table\n";
    
    foreach my $key (sort keys %groups)
    {
        print GROUPS "$key, $groups{$key}\n";
    }
    close(GROUPS);
}

sub clean_exit
{
    my ($sig) = @_;
    
    $SIG{'INT'} = $SIG{'QUIT'} = 'IGNORE';
    update_group_table;
    $nntp->quit() if (defined($nntp));
    die "Asked to quit on $sig";
}

sub print_usage
{
    print <<EOF
Usage: 
        $0 [-d] [-n|-r group] [-s server] [-h] \
              [-f] [-c] [-d directory]
Where:

-a           Annotate download
-g           Show debugging information (implies annotate)
-n group     Add new group to the download list
-r group     Delete group from the download list
-f           Resets the message ids for an existing group
-d directory Save files/structure in specified directory
-c           Store messages as group/msgid not group.msgid
-s server    Specify a different NNTP server

-h           Show this help information
EOF
    ;
    exit(0);
}
